"""Hand-curated search queries for benchmarking."""

from cfabric_benchmarks.queries.curated import (
    CORPUS_QUERIES,
    get_bhsa_queries,
    get_queries_for_corpus,
)

__all__ = [
    "CORPUS_QUERIES",
    "get_bhsa_queries",
    "get_queries_for_corpus",
]
